@@ -9,7 +9,9 @@
 
 #include "gtkfishbowl.h"
 #include "gtkgears.h"
-#include "gskshaderpaintable.h"
+
+#include "nodewidget.h"
+#include "graphwidget.h"
 
 const char *const css =
 ".blurred-button {"
@@ -149,38 +151,6 @@ create_switch (void)
   return w;
 }
 
-static gboolean
-update_paintable (GtkWidget     *widget,
-                  GdkFrameClock *frame_clock,
-                  gpointer       user_data)
-{
-  GskShaderPaintable *paintable;
-  gint64 frame_time;
-
-  paintable = GSK_SHADER_PAINTABLE (gtk_picture_get_paintable (GTK_PICTURE (widget)));
-  frame_time = gdk_frame_clock_get_frame_time (frame_clock);
-  gsk_shader_paintable_update_time (paintable, 0, frame_time);
-
-  return G_SOURCE_CONTINUE;
-}
-
-static GtkWidget *
-create_cogs (void)
-{
-  GtkWidget *picture;
-  static GskGLShader *cog_shader = NULL;
-  GdkPaintable *paintable;
-
- if (cog_shader == NULL)
-    cog_shader = gsk_gl_shader_new_from_resource ("/gltransition/cogs2.glsl");
-  paintable = gsk_shader_paintable_new (g_object_ref (cog_shader), NULL);
-  picture = gtk_picture_new_for_paintable (paintable);
-  gtk_widget_set_size_request (picture, 150, 75);
-  gtk_widget_add_tick_callback (picture, update_paintable, NULL, NULL);
-
-  return picture;
-}
-
 static void
 mapped (GtkWidget *w)
 {
@@ -201,36 +171,54 @@ create_menu_button (void)
   return w;
 }
 
+static GtkWidget *
+create_tiger (void)
+{
+  return node_widget_new ("/fishbowl/tiger.node");
+}
+
+static GtkWidget *
+create_graph (void)
+{
+  return graph_widget_new ();
+}
+
 static const struct {
   const char *name;
-  GtkWidget * (*create_func) (void);
+  GtkWidget * (* create_func) (void);
+  gboolean    (* check) (GtkFishbowl *fb);
 } widget_types[] = {
-  { "Icon",       create_icon           },
-  { "Button",     create_button         },
-  { "Blurbutton", create_blurred_button },
-  { "Fontbutton", create_font_button    },
-  { "Levelbar",   create_level_bar      },
-  { "Label",      create_label          },
-  { "Spinner",    create_spinner        },
-  { "Spinbutton", create_spinbutton     },
-  { "Video",      create_video          },
-  { "Gears",      create_gears          },
-  { "Switch",     create_switch         },
-  { "Menubutton", create_menu_button    },
-  { "Shader",     create_cogs           },
+  { "Icon",       create_icon,           NULL },
+  { "Button",     create_button,         NULL },
+  { "Blurbutton", create_blurred_button, NULL },
+  { "Fontbutton", create_font_button,    NULL },
+  { "Levelbar",   create_level_bar,      NULL },
+  { "Label",      create_label,          NULL },
+  { "Spinner",    create_spinner,        NULL },
+  { "Spinbutton", create_spinbutton,     NULL },
+  { "Video",      create_video,          NULL },
+  { "Gears",      create_gears,          NULL },
+  { "Switch",     create_switch,         NULL },
+  { "Menubutton", create_menu_button,    NULL },
+  { "Tiger",      create_tiger,          NULL },
+  { "Graph",      create_graph,          NULL },
 };
 
 static int selected_widget_type = -1;
 static const int N_WIDGET_TYPES = G_N_ELEMENTS (widget_types);
 
-static void
+static gboolean
 set_widget_type (GtkFishbowl *fishbowl,
                  int          widget_type_index)
 {
   GtkWidget *window;
 
   if (widget_type_index == selected_widget_type)
-    return;
+    return TRUE;
+
+  if (widget_types[widget_type_index].check != NULL &&
+      !widget_types[widget_type_index].check (fishbowl))
+    return FALSE;
 
   selected_widget_type = widget_type_index;
 
@@ -240,6 +228,8 @@ set_widget_type (GtkFishbowl *fishbowl,
   window = GTK_WIDGET (gtk_widget_get_root (GTK_WIDGET (fishbowl)));
   gtk_window_set_title (GTK_WINDOW (window),
                         widget_types[selected_widget_type].name);
+
+  return TRUE;
 }
 
 G_MODULE_EXPORT void
@@ -247,14 +237,17 @@ fishbowl_next_button_clicked_cb (GtkButton *source,
                                  gpointer   user_data)
 {
   GtkFishbowl *fishbowl = user_data;
-  int new_index;
+  int new_index = selected_widget_type;
 
-  if (selected_widget_type + 1 >= N_WIDGET_TYPES)
-    new_index = 0;
-  else
-    new_index = selected_widget_type + 1;
+  do
+    {
+      if (new_index + 1 >= N_WIDGET_TYPES)
+        new_index = 0;
+      else
+        new_index = new_index + 1;
 
-  set_widget_type (fishbowl, new_index);
+    }
+  while (!set_widget_type (fishbowl, new_index));
 }
 
 G_MODULE_EXPORT void
@@ -262,14 +255,18 @@ fishbowl_prev_button_clicked_cb (GtkButton *source,
                                  gpointer   user_data)
 {
   GtkFishbowl *fishbowl = user_data;
-  int new_index;
+  int new_index = selected_widget_type;
 
-  if (selected_widget_type - 1 < 0)
-    new_index = N_WIDGET_TYPES - 1;
-  else
-    new_index = selected_widget_type - 1;
+  do
+    {
+      if (new_index - 1 < 0)
+        new_index = N_WIDGET_TYPES - 1;
+      else
+        new_index = new_index - 1;
 
-  set_widget_type (fishbowl, new_index);
+    }
+
+  while (!set_widget_type (fishbowl, new_index));
 }
 
 G_MODULE_EXPORT void
