@@ -77,6 +77,9 @@ gdk_macos_display_get_setting (GdkDisplay  *display,
                                const char *setting,
                                GValue      *value)
 {
+  if (gdk_display_get_debug_flags (display) & GDK_DEBUG_DEFAULT_SETTINGS)
+    return FALSE;
+
   return _gdk_macos_display_get_setting (GDK_MACOS_DISPLAY (display), setting, value);
 }
 
@@ -267,13 +270,6 @@ gdk_macos_display_get_next_serial (GdkDisplay *display)
   return ++serial;
 }
 
-static gboolean
-gdk_macos_display_has_pending (GdkDisplay *display)
-{
-  return _gdk_event_queue_find_first (display) ||
-         _gdk_macos_event_source_check_pending ();
-}
-
 static void
 gdk_macos_display_notify_startup_complete (GdkDisplay  *display,
                                            const char *startup_notification_id)
@@ -608,7 +604,6 @@ gdk_macos_display_class_init (GdkMacosDisplayClass *klass)
   display_class->get_next_serial = gdk_macos_display_get_next_serial;
   display_class->get_name = gdk_macos_display_get_name;
   display_class->get_setting = gdk_macos_display_get_setting;
-  display_class->has_pending = gdk_macos_display_has_pending;
   display_class->init_gl = gdk_macos_display_init_gl;
   display_class->notify_startup_complete = gdk_macos_display_notify_startup_complete;
   display_class->queue_events = gdk_macos_display_queue_events;
@@ -991,17 +986,41 @@ _gdk_macos_display_warp_pointer (GdkMacosDisplay *self,
   CGWarpMouseCursorPosition ((CGPoint) { x, y });
 }
 
+/* Find the matching `NSEvent` for an `GdkEvent`. This function
+ * return the `NSEvent`, also for rewritten `GdkEvent`'s.
+ */
 NSEvent *
-_gdk_macos_display_get_nsevent (GdkEvent *event)
+_gdk_macos_display_get_matching_nsevent (GdkEvent *event)
 {
-  for (const GList *iter = event_map.head; iter; iter = iter->next)
+  for (GList *iter = event_map.head; iter; iter = iter->next)
     {
-      const GdkToNSEventMap *map = iter->data;
+      GdkToNSEventMap *map = iter->data;
 
       if (map->gdk_event->event_type == event->event_type &&
           map->gdk_event->device == event->device &&
           map->gdk_event->time == event->time)
-        return map->nsevent;
+        {
+          return map->nsevent;
+        }
+    }
+
+  return NULL;
+}
+
+/* Find the matching `NSEvent` for the original `GdkEvent`.
+ * If an event was rewritten, it returns `NULL`.
+ */
+NSEvent *
+_gdk_macos_display_get_exact_nsevent (GdkEvent *event)
+{
+  for (GList *iter = event_map.head; iter; iter = iter->next)
+    {
+      GdkToNSEventMap *map = iter->data;
+
+      if (map->gdk_event == event)
+        {
+          return map->nsevent;
+        }
     }
 
   return NULL;
