@@ -36,9 +36,6 @@
 #define INITGUID
 #endif
 
-/* For C-style COM wrapper macros */
-#define COBJMACROS
-
 #include "gdkdropprivate.h"
 
 #include "gdkdrag.h"
@@ -133,7 +130,7 @@ struct _drop_target_context
    */
   GdkSurface                     *surface;
   /* This is given to us by the OS, we store it here
-   * until the drag leaves our window. It is referenced
+   * until the drag leaves our surface HWND. It is referenced
    * (using COM reference counting).
    */
   IDataObject                    *data_object;
@@ -269,7 +266,8 @@ idroptarget_release (LPDROPTARGET This)
 }
 
 static GdkContentFormats *
-query_object_formats (LPDATAOBJECT  pDataObj,
+query_object_formats (GdkDisplay   *display,
+                      LPDATAOBJECT  pDataObj,
                       GArray       *w32format_contentformat_map)
 {
   IEnumFORMATETC *pfmt = NULL;
@@ -298,12 +296,11 @@ query_object_formats (LPDATAOBJECT  pDataObj,
         GDK_NOTE (DND, g_print ("supported unnamed? source format 0x%x\n", fmt.cfFormat));
 
       g_free (registered_name);
-      _gdk_win32_add_w32format_to_pairs (fmt.cfFormat, w32format_contentformat_map, builder);
+      gdk_win32_clipdrop_add_win32_format_to_pairs (gdk_win32_display_get_clipdrop (display), fmt.cfFormat, w32format_contentformat_map, builder);
       hr = IEnumFORMATETC_Next (pfmt, 1, &fmt, NULL);
     }
 
-  if (pfmt)
-    IEnumFORMATETC_Release (pfmt);
+  gdk_win32_com_clear (&pfmt);
 
   result_formats = gdk_content_formats_builder_free_to_formats (builder);
 
@@ -313,8 +310,7 @@ query_object_formats (LPDATAOBJECT  pDataObj,
 static void
 set_data_object (LPDATAOBJECT *location, LPDATAOBJECT data_object)
 {
-  if (*location != NULL)
-    IDataObject_Release (*location);
+  gdk_win32_com_clear (location);
 
   *location = data_object;
 
@@ -476,7 +472,7 @@ idroptarget_dragenter (LPDROPTARGET This,
   GArray *droptarget_w32format_contentformat_map;
 
   GDK_NOTE (DND, g_print ("idroptarget_dragenter %p @ %ld : %ld"
-                          " for dest window 0x%p"
+                          " for dest surface 0x%p"
                           ". dwOKEffects = %lu\n",
                           This, pt.x, pt.y,
                           ctx->surface,
@@ -490,12 +486,12 @@ idroptarget_dragenter (LPDROPTARGET This,
   drag = NULL;
 
   if (ctx->surface)
-    drag = _gdk_win32_find_drag_for_dest_window (GDK_SURFACE_HWND (ctx->surface));
+    drag = gdk_win32_find_drag_for_dest_surface (ctx->surface);
 
   display = gdk_surface_get_display (ctx->surface);
 
   droptarget_w32format_contentformat_map = g_array_new (FALSE, FALSE, sizeof (GdkWin32ContentFormatPair));
-  formats = query_object_formats (pDataObj, droptarget_w32format_contentformat_map);
+  formats = query_object_formats (display, pDataObj, droptarget_w32format_contentformat_map);
   drop = gdk_drop_new (display,
                        gdk_seat_get_pointer (gdk_display_get_default_seat (display)),
                        drag,
@@ -537,7 +533,7 @@ idroptarget_dragenter (LPDROPTARGET This,
 
 /* NOTE: This method is called continuously, even if nothing is
  * happening, as long as the drag operation is in progress and
- * the cursor is above our window.
+ * the cursor is above our surface.
  * It is OK to return a "safe" dropeffect value (DROPEFFECT_NONE,
  * to indicate that the drop is not possible here), when we
  * do not yet have any real information about acceptability of
@@ -685,7 +681,7 @@ static IDropTargetVtbl idt_vtbl = {
 };
 
 static drop_target_context *
-target_context_new (GdkSurface *window)
+target_context_new (GdkSurface *surface)
 {
   drop_target_context *result;
 
@@ -693,11 +689,11 @@ target_context_new (GdkSurface *window)
   result->idt.lpVtbl = &idt_vtbl;
   result->ref_count = 0;
 
-  result->surface = window;
+  result->surface = surface;
 
   idroptarget_addref (&result->idt);
 
-  GDK_NOTE (DND, g_print ("target_context_new: %p (window %p)\n", result, result->surface));
+  GDK_NOTE (DND, g_print ("target_context_new: %p (surface %p)\n", result, result->surface));
 
   return result;
 }
@@ -880,35 +876,35 @@ gdk_destroy_filter (GdkXEvent *xev,
 #endif
 
 void
-_gdk_win32_surface_register_dnd (GdkSurface *window)
+_gdk_win32_surface_register_dnd (GdkSurface *surface)
 {
   drop_target_context *ctx;
   HRESULT hr;
   GdkWin32Surface *impl;
 
-  g_return_if_fail (window != NULL);
+  g_return_if_fail (surface != NULL);
 
-  if (g_object_get_data (G_OBJECT (window), "gdk-dnd-registered") != NULL)
+  if (g_object_get_data (G_OBJECT (surface), "gdk-dnd-registered") != NULL)
     return;
   else
-    g_object_set_data (G_OBJECT (window), "gdk-dnd-registered", GINT_TO_POINTER (TRUE));
+    g_object_set_data (G_OBJECT (surface), "gdk-dnd-registered", GINT_TO_POINTER (TRUE));
 
-  GDK_NOTE (DND, g_print ("gdk_win32_surface_register_dnd: %p\n", GDK_SURFACE_HWND (window)));
+  GDK_NOTE (DND, g_print ("gdk_win32_surface_register_dnd: %p\n", GDK_SURFACE_HWND (surface)));
 
-  impl = GDK_WIN32_SURFACE (window);
+  impl = GDK_WIN32_SURFACE (surface);
 
-  /* Return if window is already setup for DND. */
+  /* Return if surface is already setup for DND. */
   if (impl->drop_target != NULL)
     return;
 
-  ctx = target_context_new (window);
+  ctx = target_context_new (surface);
 
   hr = CoLockObjectExternal ((IUnknown *) &ctx->idt, TRUE, FALSE);
   if (!SUCCEEDED (hr))
     OTHER_API_FAILED ("CoLockObjectExternal");
   else
     {
-      hr = RegisterDragDrop (GDK_SURFACE_HWND (window), &ctx->idt);
+      hr = RegisterDragDrop (GDK_SURFACE_HWND (surface), &ctx->idt);
       if (hr == DRAGDROP_E_ALREADYREGISTERED)
         {
           g_print ("DRAGDROP_E_ALREADYREGISTERED\n");
@@ -924,9 +920,9 @@ _gdk_win32_surface_register_dnd (GdkSurface *window)
 }
 
 void
-_gdk_win32_surface_unregister_dnd (GdkSurface *window)
+_gdk_win32_surface_unregister_dnd (GdkSurface *surface)
 {
-  GdkWin32Surface *impl = GDK_WIN32_SURFACE (window);
+  GdkWin32Surface *impl = GDK_WIN32_SURFACE (surface);
 
   if (impl->drop_target)
     idroptarget_release (&impl->drop_target->idt);
@@ -1016,6 +1012,7 @@ gdk_win32_drop_read_async (GdkDrop             *drop,
         {
           g_task_return_new_error (task, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
                                    _("No compatible transfer format found"));
+          g_object_unref (task);
           g_clear_pointer (&drop_win32->dropfiles_list, g_free);
 
           return;
@@ -1025,6 +1022,7 @@ gdk_win32_drop_read_async (GdkDrop             *drop,
       drop_win32->dropfiles_list = NULL;
       g_object_set_data (G_OBJECT (stream), "gdk-dnd-stream-contenttype", (gpointer) "text/uri-list");
       g_task_return_pointer (task, stream, g_object_unref);
+      g_object_unref (task);
 
       return;
     }
@@ -1035,6 +1033,7 @@ gdk_win32_drop_read_async (GdkDrop             *drop,
     {
       g_task_return_new_error (task, G_IO_ERROR, G_IO_ERROR_FAILED,
                                _("GDK surface 0x%p is not registered as a drop target"), gdk_drop_get_surface (drop));
+      g_object_unref (task);
       return;
     }
 
@@ -1042,6 +1041,7 @@ gdk_win32_drop_read_async (GdkDrop             *drop,
     {
       g_task_return_new_error (task, G_IO_ERROR, G_IO_ERROR_FAILED,
                                _("Target context record 0x%p has no data object"), tctx);
+      g_object_unref (task);
       return;
     }
 
@@ -1061,6 +1061,7 @@ gdk_win32_drop_read_async (GdkDrop             *drop,
     {
       g_task_return_new_error (task, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
                                _("No compatible transfer format found"));
+      g_object_unref (task);
       return;
     }
 
@@ -1080,6 +1081,7 @@ gdk_win32_drop_read_async (GdkDrop             *drop,
     {
       g_task_return_new_error (task, G_IO_ERROR, G_IO_ERROR_FAILED,
                                _("IDataObject_GetData (0x%x) failed, returning 0x%lx"), fmt.cfFormat, hr);
+      g_object_unref (task);
       return;
     }
 
@@ -1092,6 +1094,7 @@ gdk_win32_drop_read_async (GdkDrop             *drop,
           if (data == NULL)
             {
               ReleaseStgMedium (&storage);
+              g_object_unref (task);
 
               return;
             }
@@ -1103,7 +1106,12 @@ gdk_win32_drop_read_async (GdkDrop             *drop,
     }
   else
     {
-      _gdk_win32_transmute_windows_data (pair->w32format, pair->contentformat, storage.hGlobal, &data, &data_len);
+      GdkDisplay *display = gdk_drop_get_display (drop);
+      gdk_win32_clipdrop_transmute_windows_data (gdk_win32_display_get_clipdrop (display),
+                                                 pair->w32format, pair->contentformat,
+                                                 storage.hGlobal,
+                                                 &data,
+                                                 &data_len);
     }
 
   ReleaseStgMedium (&storage);
@@ -1112,12 +1120,14 @@ gdk_win32_drop_read_async (GdkDrop             *drop,
     {
       g_task_return_new_error (task, G_IO_ERROR, G_IO_ERROR_FAILED,
                                _("Failed to transmute DnD data W32 format 0x%x to %p (%s)"), pair->w32format, pair->contentformat, pair->contentformat);
+      g_object_unref (task);
       return;
     }
 
   stream = g_memory_input_stream_new_from_data (data, data_len, g_free);
   g_object_set_data (G_OBJECT (stream), "gdk-dnd-stream-contenttype", (gpointer) pair->contentformat);
   g_task_return_pointer (task, stream, g_object_unref);
+  g_object_unref (task);
 }
 
 static GInputStream *
