@@ -75,6 +75,8 @@
 #include <X11/extensions/Xrandr.h>
 #endif
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+
 enum {
   XEVENT,
   LAST_SIGNAL
@@ -113,6 +115,8 @@ static void gdk_internal_connection_watch (Display  *display,
 					   gboolean  opening,
 					   XPointer *watch_data);
 
+static void update_shadow_width (GdkDisplay *display);
+
 typedef struct _GdkEventTypeX11 GdkEventTypeX11;
 
 struct _GdkEventTypeX11
@@ -668,6 +672,7 @@ gdk_x11_display_translate_event (GdkEventTranslator *translator,
           g_free (x11_screen->window_manager_name);
           x11_screen->window_manager_name = g_strdup ("unknown");
 
+          update_shadow_width (display);
           /* careful, reentrancy */
           _gdk_x11_screen_window_manager_changed (x11_screen);
 
@@ -1415,6 +1420,8 @@ gdk_x11_display_init_leader_surface (GdkX11Display *self)
  * returned.
  *
  * Returns: (nullable) (transfer full): The new display
+ *
+ * Deprecated: 4.18
  */
 GdkDisplay *
 gdk_x11_display_open (const char *display_name)
@@ -1428,9 +1435,8 @@ gdk_x11_display_open (const char *display_name)
   int ignore;
   int maj, min;
   char *cm_name;
-  gboolean frame_extents;
-  gboolean rgba;
-  gboolean composited;
+
+  gdk_ensure_initialized ();
 
   XInitThreads ();
 
@@ -1648,16 +1654,26 @@ gdk_x11_display_open (const char *display_name)
                                                   gdk_x11_get_xatom_by_name_for_display (display, cm_name)) != None);
   g_free (cm_name);
 
+  update_shadow_width (display);
+
+  gdk_display_emit_opened (display);
+
+  return display;
+}
+
+static void
+update_shadow_width (GdkDisplay *display)
+{
+  gboolean frame_extents;
+  gboolean rgba;
+  gboolean composited;
+
   frame_extents = gdk_x11_screen_supports_net_wm_hint (gdk_x11_display_get_screen (display),
                                                        g_intern_static_string ("_GTK_FRAME_EXTENTS"));
   rgba = gdk_display_is_rgba (display);
   composited = gdk_display_is_composited (display);
 
   gdk_display_set_shadow_width (display, frame_extents && rgba && composited);
-
-  gdk_display_emit_opened (display);
-
-  return display;
 }
 
 /**
@@ -1850,12 +1866,6 @@ gdk_x11_display_flush (GdkDisplay *display)
     XFlush (GDK_DISPLAY_XDISPLAY (display));
 }
 
-static gboolean
-gdk_x11_display_has_pending (GdkDisplay *display)
-{
-  return XPending (GDK_DISPLAY_XDISPLAY (display));
-}
-
 /**
  * gdk_x11_display_get_default_group:
  * @display: (type GdkX11Display): a `GdkDisplay`
@@ -3058,7 +3068,6 @@ gdk_x11_display_class_init (GdkX11DisplayClass * class)
   display_class->sync = gdk_x11_display_sync;
   display_class->flush = gdk_x11_display_flush;
   display_class->make_default = gdk_x11_display_make_default;
-  display_class->has_pending = gdk_x11_display_has_pending;
   display_class->queue_events = _gdk_x11_display_queue_events;
   display_class->get_app_launch_context = _gdk_x11_display_get_app_launch_context;
 
