@@ -29,16 +29,21 @@
 #include "gtkprivate.h"
 #include "gtkshortcut.h"
 #include "gtkshortcuttrigger.h"
+#include "gtkpopovermenu.h"
 #include "gtkwidgetprivate.h"
 #include "gtkeventcontrollerfocus.h"
 
+#include <glib/gi18n-lib.h>
+
 /**
  * GtkEditableLabel:
  *
- * A `GtkEditableLabel` is a label that allows users to
- * edit the text by switching to an “edit mode”.
+ * Allows users to edit the displayed text by switching to an “edit mode”.
  *
- * ![An example GtkEditableLabel](editable-label.png)
+ * <picture>
+ *   <source srcset="editable-label-dark.png" media="(prefers-color-scheme: dark)">
+ *   <img alt="An example GtkEditableLabel" src="editable-label.png">
+ * </picture>
  *
  * `GtkEditableLabel` does not have API of its own, but it
  * implements the [iface@Gtk.Editable] interface.
@@ -48,6 +53,20 @@
  * for leaving the edit mode are the Enter key (to save
  * the results) or the Escape key (to cancel the editing).
  *
+ * # Shortcuts and Gestures
+ *
+ * `GtkEditableLabel` supports the following keyboard shortcuts:
+ *
+ * - <kbd>Enter</kbd> starts editing.
+ * - <kbd>Escape</kbd> stops editing.
+ *
+ * # Actions
+ *
+ * `GtkEditableLabel` defines a set of built-in actions:
+ *
+ * - `editing.starts` switches the widget into editing mode.
+ * - `editing.stop` switches the widget out of editing mode.
+ *
  * # CSS nodes
  *
  * ```
@@ -72,6 +91,7 @@ struct _GtkEditableLabel
   GtkWidget *stack;
   GtkWidget *label;
   GtkWidget *entry;
+  GtkWidget *popup_menu;
 
   guint stop_editing_soon_id;
 };
@@ -126,9 +146,86 @@ stop_editing (GtkWidget  *widget,
 }
 
 static void
-clicked_cb (GtkWidget *self)
+clipboard_copy (GtkWidget  *widget,
+                const char *name,
+                GVariant   *parameter)
 {
-  gtk_widget_activate_action (self, "editing.start", NULL);
+  GtkEditableLabel *self = GTK_EDITABLE_LABEL (widget);
+  GdkClipboard *clipboard;
+  const char *text;
+
+  clipboard = gtk_widget_get_clipboard (widget);
+  text = gtk_label_get_label (GTK_LABEL (self->label));
+
+  gdk_clipboard_set_text (clipboard, text);
+}
+
+static GMenuModel *
+create_menu (void)
+{
+  GMenu *menu;
+
+  menu = g_menu_new ();
+
+  g_menu_append (menu, _("_Edit"), "editing.start");
+  g_menu_append (menu, _("_Copy"), "clipboard.copy");
+
+  return G_MENU_MODEL (menu);
+}
+
+static void
+do_popup (GtkEditableLabel *self,
+          double            x,
+          double            y)
+{
+  if (x != -1 && y != -1)
+    {
+      GdkRectangle rect = { x, y, 1, 1 };
+      gtk_popover_set_pointing_to (GTK_POPOVER (self->popup_menu), &rect);
+    }
+  else
+    {
+      gtk_popover_set_pointing_to (GTK_POPOVER (self->popup_menu), NULL);
+    }
+
+  gtk_popover_popup (GTK_POPOVER (self->popup_menu));
+}
+
+static void
+clicked_cb (GtkGestureClick  *gesture,
+            int               n_press,
+            double            x,
+            double            y,
+            GtkEditableLabel *self)
+{
+  guint button;
+  GdkEventSequence *sequence;
+  GdkEvent *event;
+
+  button = gtk_gesture_single_get_current_button (GTK_GESTURE_SINGLE (gesture));
+  sequence = gtk_gesture_single_get_current_sequence (GTK_GESTURE_SINGLE (gesture));
+  event = gtk_gesture_get_last_event (GTK_GESTURE (gesture), sequence);
+
+  if (button == GDK_BUTTON_PRIMARY)
+    {
+      gtk_gesture_set_state (GTK_GESTURE (gesture), GTK_EVENT_SEQUENCE_CLAIMED);
+      gtk_widget_activate_action (GTK_WIDGET (self), "editing.start", NULL);
+    }
+  else if (gdk_event_triggers_context_menu (event))
+    {
+      gtk_gesture_set_state (GTK_GESTURE (gesture), GTK_EVENT_SEQUENCE_CLAIMED);
+      do_popup (self, x, y);
+    }
+}
+
+static void
+popup_menu (GtkWidget  *widget,
+            const char *action_name,
+            GVariant   *parameters)
+{
+  GtkEditableLabel *self = GTK_EDITABLE_LABEL (widget);
+
+  do_popup (self, -1, -1);
 }
 
 static void
@@ -212,6 +309,9 @@ static void
 gtk_editable_label_focus_out (GtkEventController *controller,
                               GtkEditableLabel   *self)
 {
+  if (!gtk_editable_label_get_editing (self))
+    return;
+
   if (self->stop_editing_soon_id == 0)
     self->stop_editing_soon_id = g_timeout_add (100, stop_editing_soon, controller);
 }
@@ -223,6 +323,7 @@ gtk_editable_label_init (GtkEditableLabel *self)
   GtkDropTarget *target;
   GtkDragSource *source;
   GtkEventController *controller;
+  GMenuModel *model;
 
   gtk_widget_set_focusable (GTK_WIDGET (self), TRUE);
 
@@ -231,13 +332,28 @@ gtk_editable_label_init (GtkEditableLabel *self)
   gtk_label_set_xalign (GTK_LABEL (self->label), 0.0);
   self->entry = gtk_text_new ();
 
+  model = create_menu ();
+  self->popup_menu = gtk_popover_menu_new_from_model (model);
+  gtk_widget_set_parent (self->popup_menu, GTK_WIDGET (self));
+  gtk_popover_set_position (GTK_POPOVER (self->popup_menu), GTK_POS_BOTTOM);
+
+  gtk_popover_set_has_arrow (GTK_POPOVER (self->popup_menu), FALSE);
+  gtk_widget_set_halign (self->popup_menu, GTK_ALIGN_START);
+
+  gtk_accessible_update_property (GTK_ACCESSIBLE (self->popup_menu),
+                                  GTK_ACCESSIBLE_PROPERTY_LABEL, _("Context menu"),
+                                  -1);
+
+  g_object_unref (model);
+
   gtk_stack_add_named (GTK_STACK (self->stack), self->label, "label");
   gtk_stack_add_named (GTK_STACK (self->stack), self->entry, "entry");
 
   gtk_widget_set_parent (self->stack, GTK_WIDGET (self));
 
   gesture = gtk_gesture_click_new ();
-  g_signal_connect_swapped (gesture, "released", G_CALLBACK (clicked_cb), self);
+  gtk_gesture_single_set_button (GTK_GESTURE_SINGLE (gesture), 0);
+  g_signal_connect (gesture, "pressed", G_CALLBACK (clicked_cb), self);
   gtk_widget_add_controller (self->label, GTK_EVENT_CONTROLLER (gesture));
 
   g_signal_connect_swapped (self->entry, "activate", G_CALLBACK (activate_cb), self);
@@ -361,6 +477,7 @@ gtk_editable_label_dispose (GObject *object)
 
   gtk_editable_finish_delegate (GTK_EDITABLE (self));
 
+  g_clear_pointer (&self->popup_menu, gtk_widget_unparent);
   g_clear_pointer (&self->stack, gtk_widget_unparent);
 
   self->entry = NULL;
@@ -387,7 +504,7 @@ gtk_editable_label_class_init (GtkEditableLabelClass *class)
   widget_class->grab_focus = gtk_editable_label_grab_focus;
 
   /**
-   * GtkEditableLabel:editing: (attributes org.gtk.Property.get=gtk_editable_label_get_editing)
+   * GtkEditableLabel:editing:
    *
    * This property is %TRUE while the widget is in edit mode.
    */
@@ -407,10 +524,10 @@ gtk_editable_label_class_init (GtkEditableLabelClass *class)
    * user can make changes to the text.
    *
    * The default bindings for this action are clicking
-   * on the widget and the Enter key.
+   * on the widget and the <kbd>Enter</kbd> key.
    *
    * This action is disabled when `GtkEditableLabel:editing`
-   * is %FALSE.
+   * is %TRUE.
    */
   gtk_widget_class_install_action (widget_class, "editing.start", NULL, start_editing);
 
@@ -445,6 +562,18 @@ gtk_editable_label_class_init (GtkEditableLabelClass *class)
                                        "editing.stop",
                                        "b", FALSE);
 
+  gtk_widget_class_install_action (widget_class, "clipboard.copy", NULL, clipboard_copy);
+  gtk_widget_class_install_action (widget_class, "menu.popup", NULL, popup_menu);
+
+  gtk_widget_class_add_binding_action (widget_class,
+                                       GDK_KEY_F10, GDK_SHIFT_MASK,
+                                       "menu.popup",
+                                       NULL);
+  gtk_widget_class_add_binding_action (widget_class,
+                                       GDK_KEY_Menu, 0,
+                                       "menu.popup",
+                                       NULL);
+
   gtk_widget_class_set_layout_manager_type (widget_class, GTK_TYPE_BIN_LAYOUT);
   gtk_widget_class_set_css_name (widget_class, "editablelabel");
 }
@@ -466,7 +595,7 @@ gtk_editable_label_new (const char *str)
 }
 
 /**
- * gtk_editable_label_get_editing: (attributes org.gtk.Method.get_property=editing)
+ * gtk_editable_label_get_editing:
  * @self: a `GtkEditableLabel`
  *
  * Returns whether the label is currently in “editing mode”.
