@@ -20,8 +20,7 @@
 /**
  * GtkEventControllerScroll:
  *
- * `GtkEventControllerScroll` is an event controller that handles scroll
- * events.
+ * Handles scroll events.
  *
  * It is capable of handling both discrete and continuous scroll
  * events from mice or touchpads, abstracting them both with the
@@ -410,16 +409,22 @@ gtk_event_controller_scroll_handle_event (GtkEventController *controller,
             }
           else
             {
-              if (ABS (scroll->cur_dx) >= 1)
+              if (ABS (scroll->cur_dx) >= 0.5)
                 {
                   steps = trunc (scroll->cur_dx);
+                  if (steps == 0)
+                    steps = (scroll->cur_dx > 0) ? 1 : -1;
+
                   scroll->cur_dx -= steps;
                   dx = steps;
                 }
 
-              if (ABS (scroll->cur_dy) >= 1)
+              if (ABS (scroll->cur_dy) >= 0.5)
                 {
                   steps = trunc (scroll->cur_dy);
+                  if (steps == 0)
+                    steps = (scroll->cur_dy > 0) ? 1 : -1;
+
                   scroll->cur_dy -= steps;
                   dy = steps;
                 }
@@ -459,16 +464,22 @@ gtk_event_controller_scroll_handle_event (GtkEventController *controller,
           scroll->cur_dy += dy;
           dx = dy = 0;
 
-          if (ABS (scroll->cur_dx) >= 1)
+          if (ABS (scroll->cur_dx) >= 0.5)
             {
               steps = trunc (scroll->cur_dx);
+              if (steps == 0)
+                steps = (scroll->cur_dx > 0) ? 1 : -1;
+
               scroll->cur_dx -= steps;
               dx = steps;
             }
 
-          if (ABS (scroll->cur_dy) >= 1)
+          if (ABS (scroll->cur_dy) >= 0.5)
             {
               steps = trunc (scroll->cur_dy);
+              if (steps == 0)
+                steps = (scroll->cur_dy > 0) ? 1 : -1;
+
               scroll->cur_dy -= steps;
               dy = steps;
             }
@@ -509,7 +520,7 @@ gtk_event_controller_scroll_class_init (GtkEventControllerScrollClass *klass)
   controller_class->handle_event = gtk_event_controller_scroll_handle_event;
 
   /**
-   * GtkEventControllerScroll:flags: (attributes org.gtk.Property.get=gtk_event_controller_scroll_get_flags org.gtk.Property.set=gtk_event_controller_scroll_set_flags)
+   * GtkEventControllerScroll:flags:
    *
    * The flags affecting event controller behavior.
    */
@@ -629,7 +640,7 @@ gtk_event_controller_scroll_new (GtkEventControllerScrollFlags flags)
 }
 
 /**
- * gtk_event_controller_scroll_set_flags: (attributes org.gtk.Method.set_property=flags)
+ * gtk_event_controller_scroll_set_flags:
  * @scroll: a `GtkEventControllerScroll`
  * @flags: flags affecting the controller behavior
  *
@@ -649,7 +660,7 @@ gtk_event_controller_scroll_set_flags (GtkEventControllerScroll      *scroll,
 }
 
 /**
- * gtk_event_controller_scroll_get_flags: (attributes org.gtk.Method.get_property=flags)
+ * gtk_event_controller_scroll_get_flags:
  * @scroll: a `GtkEventControllerScroll`
  *
  * Gets the flags conditioning the scroll controller behavior.
