@@ -737,6 +737,16 @@ gtk_menu_tracker_item_get_accel (GtkMenuTrackerItem *self)
   return gtk_action_muxer_get_primary_accel (GTK_ACTION_MUXER (self->observable), self->action_and_target);
 }
 
+const char *
+gtk_menu_tracker_item_get_action_name (GtkMenuTrackerItem *self)
+{
+
+  if (!self->action_and_target)
+    return NULL;
+
+  return strrchr (self->action_and_target, '|') + 1;
+}
+
 const char *
 gtk_menu_tracker_item_get_special (GtkMenuTrackerItem *self)
 {
