@@ -21,6 +21,7 @@
 
 #include <gtk/gtk.h>
 #include <gsk/gskroundedrectprivate.h>
+#include <gsk/gskrectprivate.h>
 
 static void
 test_contains_rect (void)
@@ -212,6 +213,23 @@ test_intersect_with_rect (void)
       ROUNDED_RECT_INIT (0, 0, 0, 0, 0), GSK_INTERSECTION_NOT_REPRESENTABLE },
     { ROUNDED_RECT_INIT_UNIFORM (-200, 0, 200, 100, 0, 0, 0, 40), GRAPHENE_RECT_INIT (-200, 0, 160, 100),
       ROUNDED_RECT_INIT_UNIFORM (-200, 0, 160, 100, 0, 0, 0, 40), GSK_INTERSECTION_NONEMPTY },
+
+    { ROUNDED_RECT_INIT_UNIFORM (0, 0, 50, 50, 50, 0, 50, 0), GRAPHENE_RECT_INIT (0, 0, 49, 49),
+      ROUNDED_RECT_INIT_UNIFORM (0, 0, 0, 0, 0, 0, 0, 0), GSK_INTERSECTION_NOT_REPRESENTABLE },
+    { ROUNDED_RECT_INIT_UNIFORM (0, 0, 50, 50, 50, 0, 50, 0), GRAPHENE_RECT_INIT (1, 0, 49, 49),
+      ROUNDED_RECT_INIT_UNIFORM (0, 0, 0, 0, 0, 0, 0, 0), GSK_INTERSECTION_NOT_REPRESENTABLE },
+    { ROUNDED_RECT_INIT_UNIFORM (0, 0, 50, 50, 50, 0, 50, 0), GRAPHENE_RECT_INIT (0, 1, 49, 49),
+      ROUNDED_RECT_INIT_UNIFORM (0, 0, 0, 0, 0, 0, 0, 0), GSK_INTERSECTION_NOT_REPRESENTABLE },
+    { ROUNDED_RECT_INIT_UNIFORM (0, 0, 50, 50, 50, 0, 50, 0), GRAPHENE_RECT_INIT (1, 1, 49, 49),
+      ROUNDED_RECT_INIT_UNIFORM (0, 0, 0, 0, 0, 0, 0, 0), GSK_INTERSECTION_NOT_REPRESENTABLE },
+    { ROUNDED_RECT_INIT_UNIFORM (0, 0, 50, 50, 0, 50, 0, 50), GRAPHENE_RECT_INIT (0, 0, 49, 49),
+      ROUNDED_RECT_INIT_UNIFORM (0, 0, 0, 0, 0, 0, 0, 0), GSK_INTERSECTION_NOT_REPRESENTABLE },
+    { ROUNDED_RECT_INIT_UNIFORM (0, 0, 50, 50, 0, 50, 0, 50), GRAPHENE_RECT_INIT (1, 0, 49, 49),
+      ROUNDED_RECT_INIT_UNIFORM (0, 0, 0, 0, 0, 0, 0, 0), GSK_INTERSECTION_NOT_REPRESENTABLE },
+    { ROUNDED_RECT_INIT_UNIFORM (0, 0, 50, 50, 0, 50, 0, 50), GRAPHENE_RECT_INIT (0, 1, 49, 49),
+      ROUNDED_RECT_INIT_UNIFORM (0, 0, 0, 0, 0, 0, 0, 0), GSK_INTERSECTION_NOT_REPRESENTABLE },
+    { ROUNDED_RECT_INIT_UNIFORM (0, 0, 50, 50, 0, 50, 0, 50), GRAPHENE_RECT_INIT (1, 1, 49, 49),
+      ROUNDED_RECT_INIT_UNIFORM (0, 0, 0, 0, 0, 0, 0, 0), GSK_INTERSECTION_NOT_REPRESENTABLE },
   };
 
   for (unsigned int i = 0; i < G_N_ELEMENTS (test); i++)
@@ -385,11 +403,154 @@ test_intersect (void)
     }
 }
 
+static void
+test_rounded_rect_dihedral (void)
+{
+  struct {
+    GdkDihedral dihedral;
+    GskRoundedRect in;
+    GskRoundedRect expected;
+  } test[] = {
+    {
+      GDK_DIHEDRAL_NORMAL,
+      ROUNDED_RECT_INIT_FULL(-50, -50, 100, 100, 0, 1, 2, 3, 4, 5, 6, 7),
+      ROUNDED_RECT_INIT_FULL(-50, -50, 100, 100, 0, 1, 2, 3, 4, 5, 6, 7),
+    },
+    {
+      GDK_DIHEDRAL_90,
+      ROUNDED_RECT_INIT_FULL(-50, -50, 100, 100, 0, 1, 2, 3, 4, 5, 6, 7),
+      ROUNDED_RECT_INIT_FULL(-50, -50, 100, 100, 7, 6, 1, 0, 3, 2, 5, 4),
+    },
+    {
+      GDK_DIHEDRAL_180,
+      ROUNDED_RECT_INIT_FULL(-50, -50, 100, 100, 0, 1, 2, 3, 4, 5, 6, 7),
+      ROUNDED_RECT_INIT_FULL(-50, -50, 100, 100, 4, 5, 6, 7, 0, 1, 2, 3),
+    },
+    {
+      GDK_DIHEDRAL_270,
+      ROUNDED_RECT_INIT_FULL(-50, -50, 100, 100, 0, 1, 2, 3, 4, 5, 6, 7),
+      ROUNDED_RECT_INIT_FULL(-50, -50, 100, 100, 3, 2, 5, 4, 7, 6, 1, 0),
+    },
+    {
+      GDK_DIHEDRAL_FLIPPED,
+      ROUNDED_RECT_INIT_FULL(-50, -50, 100, 100, 0, 1, 2, 3, 4, 5, 6, 7),
+      ROUNDED_RECT_INIT_FULL(-50, -50, 100, 100, 2, 3, 0, 1, 6, 7, 4, 5),
+    },
+    {
+      GDK_DIHEDRAL_FLIPPED_90,
+      ROUNDED_RECT_INIT_FULL(-50, -50, 100, 100, 0, 1, 2, 3, 4, 5, 6, 7),
+      ROUNDED_RECT_INIT_FULL(-50, -50, 100, 100, 1, 0, 7, 6, 5, 4, 3, 2),
+    },
+    {
+      GDK_DIHEDRAL_FLIPPED_180,
+      ROUNDED_RECT_INIT_FULL(-50, -50, 100, 100, 0, 1, 2, 3, 4, 5, 6, 7),
+      ROUNDED_RECT_INIT_FULL(-50, -50, 100, 100, 6, 7, 4, 5, 2, 3, 0, 1),
+    },
+    {
+      GDK_DIHEDRAL_FLIPPED_270,
+      ROUNDED_RECT_INIT_FULL(-50, -50, 100, 100, 0, 1, 2, 3, 4, 5, 6, 7),
+      ROUNDED_RECT_INIT_FULL(-50, -50, 100, 100, 5, 4, 3, 2, 1, 0, 7, 6),
+    },
+  };
+
+  g_test_summary ("Verifies the results of gsk_rounded_rect_dihedral");
+
+  for (gsize i = 0; i < G_N_ELEMENTS (test); i++)
+    {
+      GskRoundedRect out;
+
+      gsk_rounded_rect_dihedral (&out, &test[i].in, test[i].dihedral);
+      if (!gsk_rounded_rect_equal (&out, &test[i].expected))
+        {
+          char *b = gsk_rounded_rect_to_string (&out);
+          char *expected = gsk_rounded_rect_to_string (&test[i].expected);
+          g_test_message ("rounded rect %s\n"
+                          "expected: %s\n"
+                          "got: %s\n",
+                          gdk_dihedral_get_name (i), expected, b);
+        }
+      g_assert_true (gsk_rounded_rect_equal (&out, &test[i].expected));
+    }
+}
+
+static void
+test_rect_dihedral (void)
+{
+  struct {
+    GdkDihedral dihedral;
+    graphene_rect_t in;
+    graphene_rect_t expected;
+  } test[] = {
+    {
+      GDK_DIHEDRAL_NORMAL,
+      GRAPHENE_RECT_INIT (0, 0, 50, 100),
+      GRAPHENE_RECT_INIT (0, 0, 50, 100),
+    },
+    {
+      GDK_DIHEDRAL_90,
+      GRAPHENE_RECT_INIT (0, 0, 50, 100),
+      GRAPHENE_RECT_INIT (-100, 0, 100, 50),
+    },
+    {
+      GDK_DIHEDRAL_180,
+      GRAPHENE_RECT_INIT (0, 0, 50, 100),
+      GRAPHENE_RECT_INIT (-50, -100, 50, 100),
+    },
+    {
+      GDK_DIHEDRAL_270,
+      GRAPHENE_RECT_INIT (0, 0, 50, 100),
+      GRAPHENE_RECT_INIT (0, -50, 100, 50),
+    },
+    {
+      GDK_DIHEDRAL_FLIPPED,
+      GRAPHENE_RECT_INIT (0, 0, 50, 100),
+      GRAPHENE_RECT_INIT (-50, 0, 50, 100),
+    },
+    {
+      GDK_DIHEDRAL_FLIPPED_90,
+      GRAPHENE_RECT_INIT (0, 0, 50, 100),
+      GRAPHENE_RECT_INIT (0, 0, 100, 50),
+    },
+    {
+      GDK_DIHEDRAL_FLIPPED_180,
+      GRAPHENE_RECT_INIT (0, 0, 50, 100),
+      GRAPHENE_RECT_INIT (0, -100, 50, 100),
+    },
+    {
+      GDK_DIHEDRAL_FLIPPED_270,
+      GRAPHENE_RECT_INIT (0, 0, 50, 100),
+      GRAPHENE_RECT_INIT (-100, -50, 100, 50),
+    },
+  };
+
+  g_test_summary ("Verifies the results of gsk_rect_dihedral");
+
+  for (gsize i = 0; i < G_N_ELEMENTS (test); i++)
+    {
+      graphene_rect_t out;
+
+      gsk_rect_dihedral (&test[i].in, test[i].dihedral, &out);
+      if (!gsk_rect_equal (&out, &test[i].expected))
+        {
+          graphene_rect_t a = test[i].expected;
+          graphene_rect_t b = out;
+          g_test_message ("rect %s\n"
+                          "expected: %f %f %f %f\n"
+                          "got: %f %f %f %f\n",
+                          gdk_dihedral_get_name (test[i].dihedral),
+                          a.origin.x, a.origin.y, a.size.width, a.size.height,
+                          b.origin.x, b.origin.y, b.size.width, b.size.height);
+        }
+      g_assert_true (gsk_rect_equal (&out, &test[i].expected));
+    }
+}
+
 int
 main (int   argc,
       char *argv[])
 {
   gtk_test_init (&argc, &argv, NULL);
+  g_test_set_nonfatal_assertions ();
 
   g_test_add_func ("/rounded-rect/contains-rect", test_contains_rect);
   g_test_add_func ("/rounded-rect/intersects-rect", test_intersects_rect);
@@ -398,6 +559,8 @@ main (int   argc,
   g_test_add_func ("/rounded-rect/to-float", test_to_float);
   g_test_add_func ("/rounded-rect/intersect-with-rect", test_intersect_with_rect);
   g_test_add_func ("/rounded-rect/intersect", test_intersect);
+  g_test_add_func ("/rounded-rect/dihedral", test_rounded_rect_dihedral);
+  g_test_add_func ("/rect/dihedral", test_rect_dihedral);
 
   return g_test_run ();
 }
