@@ -20,6 +20,7 @@
 #include "config.h"
 
 #include <gtk/gtk.h>
+#include "gsk/gsktransformprivate.h"
 
 #define EPSILON (1.f / 1024 / 32) /* 2^-15 */
 
@@ -406,37 +407,10 @@ test_print_parse (void)
             }
         }
     }
-}
 
-static void
-gsk_matrix_transform_rect (const graphene_matrix_t *m,
-                           const graphene_rect_t   *r,
-                           graphene_quad_t         *res)
-{
-  graphene_point_t ret[4];
-  graphene_rect_t rr;
-
-  graphene_rect_normalize_r (r, &rr);
-
-#define TRANSFORM_POINT(matrix, rect, corner, out_p)   do {\
-  graphene_vec4_t __s; \
-  graphene_point_t __p; \
-  float w; \
-  graphene_rect_get_ ## corner (rect, &__p); \
-  graphene_vec4_init (&__s, __p.x, __p.y, 0.f, 1.f); \
-  graphene_matrix_transform_vec4 (matrix, &__s, &__s); \
-  w = graphene_vec4_get_w (&__s); \
-  out_p.x = graphene_vec4_get_x (&__s) / w; \
-  out_p.y = graphene_vec4_get_y (&__s) / w;           } while (0)
-
-  TRANSFORM_POINT (m, &rr, top_left, ret[0]);
-  TRANSFORM_POINT (m, &rr, top_right, ret[1]);
-  TRANSFORM_POINT (m, &rr, bottom_right, ret[2]);
-  TRANSFORM_POINT (m, &rr, bottom_left, ret[3]);
-
-#undef TRANSFORM_POINT
-
-  graphene_quad_init (res, &ret[0], &ret[1], &ret[2], &ret[3]);
+  ret = gsk_transform_parse ("rotate(45);", &parsed);
+  g_assert_false (ret);
+  g_assert_null (parsed);
 }
 
 /* This is an auxiliary function used in the GL renderer to
@@ -582,6 +556,14 @@ test_transform_bounds (void)
   t = gsk_transform_translate (t, &GRAPHENE_POINT_INIT (-50, -50));
   gsk_transform_transform_bounds (t, &bounds, &out);
   g_assert_true (graphene_rect_equal (&out, &GRAPHENE_RECT_INIT(0, 0, 100, 100)));
+
+  t = gsk_transform_scale (NULL, -1, 1);
+  gsk_transform_transform_bounds (t, &bounds, &out);
+  g_assert_true (graphene_rect_equal (&out, &GRAPHENE_RECT_INIT(-100, 0, 100, 100)));
+
+  t = gsk_transform_scale (NULL, -1, -2);
+  gsk_transform_transform_bounds (t, &bounds, &out);
+  g_assert_true (graphene_rect_equal (&out, &GRAPHENE_RECT_INIT(-100, -200, 100, 200)));
 }
 
 #define DEG_TO_RAD(x) ((x) / 180.0 * G_PI)
@@ -727,16 +709,37 @@ test_to_2d_components (void)
 
   gsk_transform_unref (transform);
   gsk_transform_unref (transform2);
+
+  transform = gsk_transform_translate (NULL, &GRAPHENE_POINT_INIT (2, 3));
+
+  gsk_transform_to_2d_components (transform,
+                                  &skew_x, &skew_y,
+                                  &scale_x, &scale_y,
+                                  &angle,
+                                  &dx, &dy);
+  g_assert_cmpfloat_with_epsilon (skew_x, 0, 0.0001);
+  g_assert_cmpfloat_with_epsilon (skew_y, 0, 0.0001);
+  g_assert_cmpfloat_with_epsilon (scale_x, 1, 0.0001);
+  g_assert_cmpfloat_with_epsilon (scale_y, 1, 0.0001);
+  g_assert_cmpfloat_with_epsilon (angle, 0, 0.0001);
+  g_assert_cmpfloat_with_epsilon (dx, 2, 0.0001);
+  g_assert_cmpfloat_with_epsilon (dy, 3, 0.0001);
+
+  gsk_transform_unref (transform);
 }
 
 static void
 test_transform_point (void)
 {
-  GskTransform *t, *t2;
+  GskTransform *t, *t2, *t3;
   graphene_point_t p;
 
   t = gsk_transform_scale (gsk_transform_translate (NULL, &GRAPHENE_POINT_INIT (1, 2)), 2, 2);
   t2 = gsk_transform_translate (gsk_transform_scale (NULL, 2, 2), &GRAPHENE_POINT_INIT (1, 2));
+  t3 = gsk_transform_rotate (NULL, 90);
+
+  gsk_transform_transform_point (NULL, &GRAPHENE_POINT_INIT (1,1), &p);
+  g_assert_true (graphene_point_equal (&p, &GRAPHENE_POINT_INIT (1, 1)));
 
   gsk_transform_transform_point (t, &GRAPHENE_POINT_INIT (1,1), &p);
   g_assert_true (graphene_point_equal (&p, &GRAPHENE_POINT_INIT (3, 4)));
@@ -744,14 +747,44 @@ test_transform_point (void)
   gsk_transform_transform_point (t2, &GRAPHENE_POINT_INIT (1,1), &p);
   g_assert_true (graphene_point_equal (&p, &GRAPHENE_POINT_INIT (4, 6)));
 
+  gsk_transform_transform_point (t3, &GRAPHENE_POINT_INIT (2,1), &p);
+  g_assert_true (graphene_point_equal (&p, &GRAPHENE_POINT_INIT (-1, 2)));
+
   gsk_transform_unref (t);
   gsk_transform_unref (t2);
+  gsk_transform_unref (t3);
+}
+
+static void
+test_scale_transform (void)
+{
+  GskTransform *t1, *t2;
+  char *string;
+  gboolean res;
+  GskTransform *x;
+
+  t1 = gsk_transform_scale (NULL, 4, 9);
+  t2 = gsk_transform_scale (gsk_transform_scale (NULL, 2, 3), 2, 3);
+
+  g_assert_true (gsk_transform_get_category (t1) == GSK_TRANSFORM_CATEGORY_2D_AFFINE);
+  g_assert_true (gsk_transform_get_category (t2) == GSK_TRANSFORM_CATEGORY_2D_AFFINE);
+  g_assert_true (gsk_transform_equal (t1, t2));
+
+  string = gsk_transform_to_string (t1);
+  res = gsk_transform_parse (string, &x);
+  g_assert_true (res);
+  g_assert_true (gsk_transform_equal (t1, x));
+
+  gsk_transform_unref (t1);
+  gsk_transform_unref (t2);
+  g_free (string);
+  gsk_transform_unref (x);
 }
 
 static void
 test_skew_transform (void)
 {
-  GskTransform *t1, *t2, *t3;
+  GskTransform *t1, *t2, *t3, *t4;
   char *string;
   gboolean res;
   GskTransform *x;
@@ -759,6 +792,7 @@ test_skew_transform (void)
   t1 = gsk_transform_skew (NULL, 30, 60);
   t2 = gsk_transform_skew (NULL, 0, 30);
   t3 = gsk_transform_skew (NULL, 0, -30);
+  t4 = gsk_transform_skew (NULL, 30, 0);
 
   g_assert_true (gsk_transform_get_category (t1) == GSK_TRANSFORM_CATEGORY_2D);
   g_assert_true (gsk_transform_get_category (t2) == GSK_TRANSFORM_CATEGORY_2D);
@@ -771,12 +805,30 @@ test_skew_transform (void)
   res = gsk_transform_parse (string, &x);
   g_assert_true (res);
   g_assert_true (gsk_transform_equal (t1, x));
+  g_free (string);
+  gsk_transform_unref (x);
+
+  string = gsk_transform_to_string (t3);
+  res = gsk_transform_parse (string, &x);
+  g_assert_true (res);
+  g_assert_true (gsk_transform_equal (t3, x));
+  g_free (string);
+  gsk_transform_unref (x);
+
+  string = gsk_transform_to_string (t4);
+  res = gsk_transform_parse (string, &x);
+  g_assert_true (res);
+  g_assert_true (gsk_transform_equal (t4, x));
+  g_free (string);
+  gsk_transform_unref (x);
+
+  t3 = gsk_transform_transform (t3, t4);
+  g_assert_true (gsk_transform_get_category (t3) == GSK_TRANSFORM_CATEGORY_2D);
 
   gsk_transform_unref (t1);
   gsk_transform_unref (t2);
   gsk_transform_unref (t3);
-  g_free (string);
-  gsk_transform_unref (x);
+  gsk_transform_unref (t4);
 }
 
 static void
@@ -803,6 +855,7 @@ static void
 test_rotate_transform (void)
 {
   GskTransform *t1, *t2, *t3;
+  graphene_vec3_t vec;
 
   t1 = gsk_transform_rotate (NULL, 60);
   t2 = gsk_transform_rotate (NULL, 20);
@@ -818,9 +871,26 @@ test_rotate_transform (void)
   t3 = gsk_transform_rotate (NULL, -60);
   g_assert_true (gsk_transform_equal (t1, t3));
 
+  gsk_transform_unref (t2);
+  t2 = gsk_transform_rotate_3d (NULL, -60, graphene_vec3_init (&vec, 0, 0, 1));
+  g_assert_true (gsk_transform_get_category (t2) == GSK_TRANSFORM_CATEGORY_2D);
+  g_assert_true (gsk_transform_equal (t1, t2));
+
   gsk_transform_unref (t1);
   gsk_transform_unref (t2);
   gsk_transform_unref (t3);
+
+  t1 = gsk_transform_rotate (NULL, 180);
+  g_assert_true (gsk_transform_get_category (t1) == GSK_TRANSFORM_CATEGORY_2D);
+  t1 = gsk_transform_rotate (t1, 180);
+  g_assert_true (gsk_transform_get_category (t1) == GSK_TRANSFORM_CATEGORY_IDENTITY);
+  g_assert_null (t1);
+
+  t1 = gsk_transform_rotate (NULL, 360);
+  g_assert_null (t1);
+
+  t1 = gsk_transform_rotate (NULL, -360);
+  g_assert_null (t1);
 }
 
 static void
@@ -843,11 +913,244 @@ test_rotate3d_transform (void)
   gsk_transform_unref (t2);
 }
 
+static void
+test_matrix_transform (void)
+{
+  GskTransform *t1, *t2;
+  float xx, yx, xy, yy, dx, dy;
+  graphene_matrix_t m;
+
+  if (!gsk_transform_parse ("matrix(2, 0, 0, 3, 1, -1)", &t1))
+    g_assert_not_reached ();
+  g_assert_true (gsk_transform_get_category (t1) == GSK_TRANSFORM_CATEGORY_2D);
+
+  gsk_transform_to_2d (t1, &xx, &yx, &xy, &yy, &dx, &dy);
+  g_assert_cmpfloat (xx, ==, 2.f);
+  g_assert_cmpfloat (yx, ==, 0.f);
+  g_assert_cmpfloat (xy, ==, 0.f);
+  g_assert_cmpfloat (yy, ==, 3.f);
+  g_assert_cmpfloat (dx, ==, 1.f);
+  g_assert_cmpfloat (dy, ==, -1.f);
+
+  graphene_matrix_init_from_float (&m,
+                                   (const float[]) { 2.f, 0.f, 0.f, 0.f,
+                                                     0.f, 3.f, 0.f, 0.f,
+                                                     0.f, 0.f, 1.f, 0.f,
+                                                     1., -1.f, 0.f, 1.f});
+  t2 = gsk_transform_matrix (NULL, &m);
+
+  g_assert_true (gsk_transform_equal (t1, t2));
+
+  t1 = gsk_transform_invert (t1);
+  t2 = gsk_transform_transform (t2, t1);
+  graphene_assert_fuzzy_transform_equal (t2, NULL, EPSILON);
+
+  gsk_transform_unref (t1);
+  gsk_transform_unref (t2);
+}
+
+static void
+test_matrix_roundtrip (void)
+{
+  GskTransform *transform;
+  gboolean res;
+  const char *input;
+  char *str;
+
+  input = "matrix(2, 3, 4, 5, 6, 7)";
+  res = gsk_transform_parse (input, &transform);
+  g_assert_true (res);
+  g_assert_true (gsk_transform_get_category (transform) >= GSK_TRANSFORM_CATEGORY_2D);
+
+  str = gsk_transform_to_string (transform);
+  g_assert_cmpstr (str, ==, input);
+  g_free (str);
+
+  input = "matrix3d(2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17)";
+  res = gsk_transform_parse (input, &transform);
+  g_assert_true (res);
+  g_assert_true (gsk_transform_get_category (transform) < GSK_TRANSFORM_CATEGORY_2D);
+
+  str = gsk_transform_to_string (transform);
+  g_assert_cmpstr (str, ==, input);
+  g_free (str);
+}
+
+static void
+test_to_dihedral (void)
+{
+  GskTransform *transform;
+  GdkDihedral dihedral;
+  float sx, sy, dx, dy;
+
+  g_test_summary ("Tests that gsk_transform_to_dihedral gives the expected results on a various transforms");
+
+  transform = gsk_transform_scale (NULL, 10.0, 5.0);
+  gsk_transform_to_dihedral (transform, &dihedral, &sx, &sy, &dx, &dy);
+  gsk_transform_unref (transform);
+
+  g_assert_true (dihedral == GDK_DIHEDRAL_NORMAL);
+  g_assert_cmpfloat (sx, ==, 10.0);
+  g_assert_cmpfloat (sy, ==, 5.0);
+  g_assert_cmpfloat (dx, ==, 0.0);
+  g_assert_cmpfloat (dy, ==, 0.0);
+
+  transform = gsk_transform_rotate (gsk_transform_translate (NULL, &GRAPHENE_POINT_INIT (10.0, 5.0)), 90);
+  gsk_transform_to_dihedral (transform, &dihedral, &sx, &sy, &dx, &dy);
+  gsk_transform_unref (transform);
+
+  g_assert_true (dihedral == GDK_DIHEDRAL_90);
+  g_assert_cmpfloat (sx, ==, 1.0);
+  g_assert_cmpfloat (sy, ==, 1.0);
+  g_assert_cmpfloat (dx, ==, 10.0);
+  g_assert_cmpfloat (dy, ==, 5.0);
+
+  transform = gsk_transform_scale (gsk_transform_translate (gsk_transform_scale (NULL, 2.0, 3.0), &GRAPHENE_POINT_INIT (10.0, 5.0)), -1, 1);
+  gsk_transform_to_dihedral (transform, &dihedral, &sx, &sy, &dx, &dy);
+  gsk_transform_unref (transform);
+
+  g_assert_true (dihedral == GDK_DIHEDRAL_FLIPPED);
+  g_assert_cmpfloat (sx, ==, 2.0);
+  g_assert_cmpfloat (sy, ==, 3.0);
+  g_assert_cmpfloat (dx, ==, 2.0 * 10.0);
+  g_assert_cmpfloat (dy, ==, 3.0 * 5.0);
+
+  transform = gsk_transform_rotate (gsk_transform_rotate (gsk_transform_scale (gsk_transform_translate (NULL, &GRAPHENE_POINT_INIT (10.0, 5.0)), 2.0, 3.0), 90), 90);
+  gsk_transform_to_dihedral (transform, &dihedral, &sx, &sy, &dx, &dy);
+  gsk_transform_unref (transform);
+
+  g_assert_true (dihedral == GDK_DIHEDRAL_180);
+  g_assert_cmpfloat (sx, ==, 2.0);
+  g_assert_cmpfloat (sy, ==, 3.0);
+  g_assert_cmpfloat (dx, ==, 10.0);
+  g_assert_cmpfloat (dy, ==, 5.0);
+
+  transform = gsk_transform_rotate (gsk_transform_scale (gsk_transform_scale (gsk_transform_translate (NULL, &GRAPHENE_POINT_INIT (10.0, 5.0)), 2.0, 3.0), -1, 1), 90);
+  gsk_transform_to_dihedral (transform, &dihedral, &sx, &sy, &dx, &dy);
+  gsk_transform_unref (transform);
+
+  g_assert_true (dihedral == GDK_DIHEDRAL_FLIPPED_90);
+  g_assert_cmpfloat (sx, ==, 2.0);
+  g_assert_cmpfloat (sy, ==, 3.0);
+  g_assert_cmpfloat (dx, ==, 10.0);
+  g_assert_cmpfloat (dy, ==, 5.0);
+
+  transform = gsk_transform_dihedral (gsk_transform_scale (gsk_transform_translate (NULL, &GRAPHENE_POINT_INIT (10.0, 5.0)), 2.0, 3.0), GDK_DIHEDRAL_FLIPPED_90);
+  gsk_transform_to_dihedral (transform, &dihedral, &sx, &sy, &dx, &dy);
+  gsk_transform_unref (transform);
+
+  g_assert_true (dihedral == GDK_DIHEDRAL_FLIPPED_90);
+  g_assert_cmpfloat (sx, ==, 2.0);
+  g_assert_cmpfloat (sy, ==, 3.0);
+  g_assert_cmpfloat (dx, ==, 10.0);
+  g_assert_cmpfloat (dy, ==, 5.0);
+}
+
+static void
+test_dihedral_matrix (void)
+{
+  g_test_summary ("Tests that gsk_transform_to_2d and gdk_dihedral_get_mat2 agree");
+
+  for (int i = 0; i <= GDK_DIHEDRAL_FLIPPED_270; i++)
+    {
+      float f[16] = { 1, 0, 0, 0,
+                      0, 1, 0, 0,
+                      0, 0, 1, 0,
+                      0, 0, 0, 1 };
+      float m[16] = { 1, 0, 0, 0,
+                      0, 1, 0, 0,
+                      0, 0, 1, 0,
+                      0, 0, 0, 1 };
+      graphene_matrix_t matrix;
+      graphene_matrix_t test;
+
+      GskTransform *transform = gsk_transform_dihedral (NULL, (GdkDihedral) i);
+      gsk_transform_to_2d (transform,
+                           &f[4 * 0 + 0], &f[4 * 0 + 1],
+                           &f[4 * 1 + 0], &f[4 * 1 + 1],
+                           &f[4 * 3 + 0], &f[4 * 3 + 1]);
+      graphene_matrix_init_from_float (&test, f);
+
+      /* Note that the xy/yx arguments are flipped in gdk_dihedral_get_mat2,
+       * compared to gsk_transform_to_2d
+       */
+      gdk_dihedral_get_mat2 ((GdkDihedral) i,
+                             &m[4 * 0 + 0], &m[4 * 1 + 0],
+                             &m[4 * 0 + 1], &m[4 * 1 + 1]);
+      graphene_matrix_init_from_float (&matrix, m);
+
+      graphene_assert_fuzzy_matrix_equal (&matrix, &test, EPSILON);
+
+      gsk_transform_unref (transform);
+    }
+}
+
+static void
+test_fine_category (void)
+{
+  GskTransform *transform;
+
+  g_test_summary ("Tests that we get the fine categories of dihedral transforms right");
+
+  transform = gsk_transform_rotate (NULL, 90);
+  g_assert_true (gsk_transform_get_fine_category (transform) == GSK_FINE_TRANSFORM_CATEGORY_2D_DIHEDRAL);
+  gsk_transform_unref (transform);
+
+  transform = gsk_transform_rotate (NULL, 180);
+  g_assert_true (gsk_transform_get_fine_category (transform) == GSK_FINE_TRANSFORM_CATEGORY_2D_DIHEDRAL);
+  gsk_transform_unref (transform);
+
+  transform = gsk_transform_rotate (NULL, 270);
+  g_assert_true (gsk_transform_get_fine_category (transform) == GSK_FINE_TRANSFORM_CATEGORY_2D_DIHEDRAL);
+  gsk_transform_unref (transform);
+
+  transform = gsk_transform_scale (NULL, -1, 1);
+  g_assert_true (gsk_transform_get_fine_category (transform) == GSK_FINE_TRANSFORM_CATEGORY_2D_NEGATIVE_AFFINE);
+  gsk_transform_unref (transform);
+
+  transform = gsk_transform_scale (NULL, 1, -1);
+  g_assert_true (gsk_transform_get_fine_category (transform) == GSK_FINE_TRANSFORM_CATEGORY_2D_NEGATIVE_AFFINE);
+  gsk_transform_unref (transform);
+
+  transform = gsk_transform_scale (NULL, -1, -1);
+  g_assert_true (gsk_transform_get_fine_category (transform) == GSK_FINE_TRANSFORM_CATEGORY_2D_NEGATIVE_AFFINE);
+  gsk_transform_unref (transform);
+
+  transform = gsk_transform_scale (gsk_transform_rotate (NULL, 90), -1, 1);
+  g_assert_true (gsk_transform_get_fine_category (transform) == GSK_FINE_TRANSFORM_CATEGORY_2D_DIHEDRAL);
+  gsk_transform_unref (transform);
+
+  transform = gsk_transform_scale (gsk_transform_rotate (NULL, 180), -1, 1);
+  g_assert_true (gsk_transform_get_fine_category (transform) == GSK_FINE_TRANSFORM_CATEGORY_2D_DIHEDRAL);
+  gsk_transform_unref (transform);
+
+  transform = gsk_transform_scale (gsk_transform_rotate (NULL, 270), -1, 1);
+  g_assert_true (gsk_transform_get_fine_category (transform) == GSK_FINE_TRANSFORM_CATEGORY_2D_DIHEDRAL);
+  gsk_transform_unref (transform);
+
+  transform = gsk_transform_scale (gsk_transform_rotate (NULL, 360), -1, 1);
+  g_assert_true (gsk_transform_get_fine_category (transform) == GSK_FINE_TRANSFORM_CATEGORY_2D_NEGATIVE_AFFINE);
+  gsk_transform_unref (transform);
+
+  transform = gsk_transform_scale (gsk_transform_rotate (NULL, 450), -1, 1);
+  g_assert_true (gsk_transform_get_fine_category (transform) == GSK_FINE_TRANSFORM_CATEGORY_2D_DIHEDRAL);
+  gsk_transform_unref (transform);
+
+  transform = gsk_transform_rotate (NULL, -450);
+  g_assert_true (gsk_transform_get_fine_category (transform) == GSK_FINE_TRANSFORM_CATEGORY_2D_DIHEDRAL);
+  gsk_transform_unref (transform);
+
+  transform = gsk_transform_dihedral (NULL, GDK_DIHEDRAL_FLIPPED_90);
+  g_assert_true (gsk_transform_get_fine_category (transform) == GSK_FINE_TRANSFORM_CATEGORY_2D_DIHEDRAL);
+  gsk_transform_unref (transform);
+}
+
 int
 main (int   argc,
       char *argv[])
 {
   gtk_test_init (&argc, &argv, NULL);
+  g_test_set_nonfatal_assertions ();
 
   g_test_add_func ("/transform/conversions/simple", test_conversions_simple);
   g_test_add_func ("/transform/conversions/transformed", test_conversions_transformed);
@@ -861,10 +1164,16 @@ main (int   argc,
   g_test_add_func ("/transform/point", test_transform_point);
   g_test_add_func ("/transform/to-2d", test_to_2d);
   g_test_add_func ("/transform/to-2d-components", test_to_2d_components);
+  g_test_add_func ("/transform/scale", test_scale_transform);
   g_test_add_func ("/transform/skew", test_skew_transform);
   g_test_add_func ("/transform/perspective", test_perspective_transform);
   g_test_add_func ("/transform/rotate", test_rotate_transform);
   g_test_add_func ("/transform/rotate3d", test_rotate3d_transform);
+  g_test_add_func ("/transform/matrix", test_matrix_transform);
+  g_test_add_func ("/transform/matrix/roundtrip", test_matrix_roundtrip);
+  g_test_add_func ("/transform/to-dihedral", test_to_dihedral);
+  g_test_add_func ("/transform/dihedral-matrix", test_dihedral_matrix);
+  g_test_add_func ("/transform/fine-category", test_fine_category);
 
   return g_test_run ();
 }
